"""Utility functions for PDF Translator."""

from pdf_translator.utils.fonts import FontManager
from pdf_translator.utils.page_range import parse_page_range

__all__ = ["FontManager", "parse_page_range"]
